{*******************************************************************************}
{                                                                               }
{ FileName     : CircImg.pas                                                    }
{                                                                               }
{ Author       : Ian Lane (Email: lanei@ideal.net.au)                           }
{                                                                               }
{ Synopsis     : A simple component which causes a graphic to circle around     }
{                inside its perimeter. It is useful for about boxes, etc.       }
{                                                                               }
{ Distribution : This component is free for public use and components may be    }
{                freely descended from it as long as credit is given to the     }
{                author.                                                        }
{                                                                               }
{                          Copyright (c) 1998 Ian Lane                          }
{                                                                               }
{*******************************************************************************}
unit CircImg;

interface

uses
  Classes, Controls, ExtCtrls, Graphics, Messages,
{$ifdef VER80}
  Menus, WinProcs, WinTypes;
{$else}
  Windows;
{$endif}

type
  TInitialCorner = (icTopLeft, icTopRight, icBottomLeft, icBottomRight);

const
  DefaultAnimate = True;
  DefaultDelay = 100;
  DefaultGranularity = 1;
  DefaultHeight = 105;
{$ifdef VER100}
  DefaultIncrementalDisplay = False;
{$endif}
  DefaultInitialCorner = icTopLeft;
  DefaultWidth = 105;

type
  TAnimationEvent = procedure(Sender : TObject; var ImageOffset : TPoint) of Object;
  TCirclingImage = class(TCustomControl)
  private
    FAnimate : Boolean;
    FGranularity : Integer;
    FImageOffset : TPoint;
{$ifdef VER100}
    FIncrementalDisplay : Boolean;
{$endif}
    FInitialCorner : TInitialCorner;
    FOnAnimate : TAnimationEvent;
{$ifdef VER100}
    FOnProgress : TProgressEvent;
{$endif}
    FPicture : TPicture;
    FPictureCache : Graphics.TBitmap;
    FTimer : TTimer;
    FUseApplicationIcon : Boolean;
    function GetDelay : Cardinal;
    function GetVersion : String;
    procedure PictureChanged(Sender : TObject);
{$ifdef VER100}
    procedure Progress(Sender : TObject; Stage : TProgressStage; PercentDone : Byte;
                       RedrawNow : Boolean; const R : TRect; const Msg : String);
{$endif}
    procedure SetAnimate(const Value : Boolean);
    procedure SetInitialCorner(const Value : TInitialCorner);
    procedure SetPicture(const Value : TPicture);
    procedure SetDelay(const Value : Cardinal);
    procedure SetUseApplicationIcon(const Value : Boolean);
    procedure TimerFired(Sender : TObject);
  protected
    procedure CMColorChanged(var Message : TMessage); message CM_COLORCHANGED;
    procedure CMIconChanged(var Message : TMessage); message CM_ICONCHANGED;
{$ifdef VER100}
    function GetPalette : HPALETTE; override;
{$endif}    
    procedure InvalidateGraphic;
    procedure Paint; override;
  public
    constructor Create(aOwner : TComponent); override;
    destructor Destroy; override;
    property Version : String read GetVersion;
  published
    property Align;
    property Animate : Boolean read FAnimate write SetAnimate default DefaultAnimate;
    property Color;
    property Delay : Cardinal read GetDelay write SetDelay default DefaultDelay;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Granularity : Integer read FGranularity write FGranularity default DefaultGranularity;
    property Height default DefaultHeight;
{$ifdef VER100}
    property IncrementalDisplay : Boolean read FIncrementalDisplay write FIncrementalDisplay
             default DefaultIncrementalDisplay;
{$endif}
    property InitialCorner : TInitialCorner read FInitialCorner write SetInitialCorner
             default DefaultInitialCorner;
    property Width default DefaultWidth;
    property ParentColor;
    property ParentShowHint;
    property Picture : TPicture read FPicture write SetPicture;
    property PopupMenu;
    property ShowHint;
    property UseApplicationIcon : Boolean read FUseApplicationIcon write SetUseApplicationIcon
                                          default False;
    property Visible;
    property OnAnimate : TAnimationEvent read FOnAnimate write FOnAnimate;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
{$ifdef VER100}
    property OnProgress : TProgressEvent read FOnProgress write FOnProgress;
{$endif}
{$ifndef VER80}
    property OnStartDrag;
{$endif}
  end;

procedure Register;

implementation

uses
  Forms;

const
  CurrentVersion = '1.3';

procedure TCirclingImage.CMColorChanged(var Message : TMessage);

begin
  Inherited;
  PictureChanged(nil)
end;

procedure TCirclingImage.CMIconChanged(var Message : TMessage);

begin
  Inherited;
  if FUseApplicationIcon then
    Picture.Icon := Application.Icon
end;

constructor TCirclingImage.Create(aOwner : TComponent);

begin
  Inherited Create(aOwner);
{$ifdef VER80}
  ControlStyle := ControlStyle + [csAcceptsControls];
{$else}
  ControlStyle := ControlStyle + [csAcceptsControls, csReplicatable];
{$endif}
  FPictureCache := Graphics.TBitmap.Create;
  FPicture := TPicture.Create;
  FPicture.OnChange := PictureChanged;
{$ifdef VER100}
  FPicture.OnProgress := Progress;
{$endif}
  FTimer := TTimer.Create(Self);
  FTimer.Enabled := False;
  FTimer.OnTimer := TimerFired;
  Animate := DefaultAnimate;
  FGranularity := DefaultGranularity;
  Height := DefaultHeight;
{$ifdef VER100}
  FIncrementalDisplay := DefaultIncrementalDisplay;
{$endif}
  InitialCorner := DefaultInitialCorner;
  Delay := DefaultDelay;
  Width := DefaultWidth
end;

destructor TCirclingImage.Destroy;

begin
  FPicture.Free;
  FPictureCache.Free;
  Inherited Destroy
end;

function TCirclingImage.GetDelay : Cardinal;

begin
  Result := FTimer.Interval
end;

function TCirclingImage.GetVersion : String;

begin
  Result := CurrentVersion
end;

{$ifdef VER100}
function TCirclingImage.GetPalette : HPALETTE;

begin
  Result := Picture.Graphic.Palette
end;
{$endif}

{ Invalidate the rectangle containing the graphic }
procedure TCirclingImage.InvalidateGraphic;

var
  r : TRect;

begin
  with r do
  begin
    TopLeft := FImageOffset;
    Right := FImageOffset.X + Picture.Width;
    Bottom := FImageOffset.Y + Picture.Height
  end;
  InvalidateRect(Handle, @r, False)
end;

procedure TCirclingImage.Paint;

var
  r : TRect;
{$ifdef VER80}
  FromRect, SubtRect : TRect;
{$endif}

begin
  { Erase any remains of the image at its previous location }
{$ifdef VER80}
  FromRect:= Canvas.ClipRect;
  SubtRect:= Rect(FImageOffset.X, FImageOffset.Y,
                  FImageOffset.X + Picture.Width,
                  FImageOffset.Y + Picture.Height);
  if SubtractRect(r, FromRect, SubtRect) then
{$else}
  if SubtractRect(r, Canvas.ClipRect,
                  Rect(FImageOffset.X, FImageOffset.Y,
                       FImageOffset.X + Picture.Width, FImageOffset.Y + Picture.Height)) then
{$endif}
  with Canvas do
  begin
    Brush.Color := Color;
    Brush.Style := bsSolid;
    FillRect(r)
  end;
  { Draw the image at its new location }
  Canvas.Draw(FImageOffset.X, FImageOffset.Y, FPictureCache);
  { At design-time, draw a dashed line around the component }
  if csDesigning in ComponentState then
    with Canvas do
    begin
      Brush.Style := bsCross;
      FrameRect(Rect(0, 0, Width, Height))
    end
end;

procedure TCirclingImage.PictureChanged(Sender : TObject);

begin
  with FPictureCache do
  begin
    Height := Picture.Height;
    Width := Picture.Width;
    Canvas.Brush.Color := Color;
    Canvas.FillRect(Rect(0, 0, Width, Height));
    Canvas.Draw(0, 0, Picture.Graphic)
  end;
  Invalidate
end;

{$ifdef VER100}
procedure TCirclingImage.Progress(Sender : TObject; Stage : TProgressStage;
                                  PercentDone : Byte; RedrawNow : Boolean;
                                  const R : TRect; const Msg : String);

begin
  if FIncrementalDisplay and RedrawNow then
  begin
    InvalidateGraphic;
    Update
  end;
  if Assigned(FOnProgress) then
    FOnProgress(Sender, Stage, PercentDone, RedrawNow, R, Msg)
end;
{$endif}

procedure TCirclingImage.SetAnimate(const Value : Boolean);

begin
  FAnimate := Value;
  if not (csDesigning in ComponentState) then
    FTimer.Enabled := Value
end;

procedure TCirclingImage.SetDelay(const Value : Cardinal);

begin
  FTimer.Interval := Value
end;

procedure TCirclingImage.SetInitialCorner(const Value : TInitialCorner);

begin
  FInitialCorner := Value;
  case FInitialCorner of
    icTopLeft     : FImageOffset := Point(0, 0);
    icTopRight    : FImageOffset := Point(Width - Picture.Width, 0);
    icBottomLeft  : FImageOffset := Point(0, Height - Picture.Height);
    icBottomRight : FImageOffset := Point(Width - Picture.Width, Height - Picture.Height)
  end;
  Invalidate
end;

procedure TCirclingImage.SetPicture(const Value : TPicture);

begin
  FUseApplicationIcon := False;
  FPicture.Assign(Value)
end;

procedure TCirclingImage.SetUseApplicationIcon(const Value : Boolean);

begin
  if Value <> FUseApplicationIcon then
  begin
    FUseApplicationIcon := Value;
    if FUseApplicationIcon then
      Picture.Icon := Application.Icon
  end
end;

procedure TCirclingImage.TimerFired(Sender : TObject);

  function BottomLimit : Integer;

  begin
    Result := ClientHeight - Picture.Height
  end;

  function LeftLimit : Integer;

  begin
    Result := 0
  end;

  function RightLimit : Integer;

  begin
    Result := ClientWidth - Picture.Width
  end;

  function TopLimit : Integer;

  begin
    Result := 0
  end;

begin { TCirclingImage.TimerFired }
  { Invalidate the graphic at its present location so that it gets erased }
  InvalidateGraphic;
  { Calculate the new position of the graphic }
  if FImageOffset.Y = TopLimit then
    if FImageOffset.X = LeftLimit then
      if Granularity > 0 then
        Inc(FImageOffset.X, Granularity)
      else
        Dec(FImageOffset.Y, Granularity)
    else if FImageOffset.X = RightLimit then
      if Granularity > 0 then
        Inc(FImageOffset.Y, Granularity)
      else
        Inc(FImageOffset.X, Granularity)
    else
      Inc(FImageOffset.X, Granularity)
  else if FImageOffset.Y = BottomLimit then
    if FImageOffset.X = LeftLimit then
      if Granularity > 0 then
        Dec(FImageOffset.Y, Granularity)
      else
        Dec(FImageOffset.X, Granularity)
    else if FImageOffset.X = RightLimit then
      if Granularity > 0 then
        Dec(FImageOffset.X, Granularity)
      else
        Inc(FImageOffset.Y, Granularity)
    else
      Dec(FImageOffset.X, Granularity)
  else
    if FImageOffset.X = LeftLimit then
      Dec(FImageOffset.Y, Granularity)
    else
      Inc(FImageOffset.Y, Granularity);
  { Constrain the graphics position to within our component }
  if FImageOffset.X < LeftLimit then
    FImageOffset.X := LeftLimit
  else if FImageOffset.X > RightLimit then
    FImageOffset.X := RightLimit;
  if FImageOffset.Y < TopLimit then
    FImageOffset.Y := TopLimit
  else if FImageOffset.Y > BottomLimit then
    FImageOffset.Y := BottomLimit;
  { Give the OnAnimate event a chance to make any changes }
  if Assigned(FOnAnimate) then
    FOnAnimate(Self, FImageOffset);
  { Invalidate the graphic at its new location so that it gets drawn }
  InvalidateGraphic
end;

procedure Register;

begin
  RegisterComponents('Extras', [TCirclingImage])
end;

end.
